@@ -12,7 +12,14 @@ from typing import Annotated, Any, Self
 from pydantic import BaseModel, BeforeValidator, ConfigDict, Field
 from pydantic.alias_generators import to_camel
 
+from .converters import (
+    device_bool_to_python,
+    div_10,
+    enum_validator,
+    tou_override_to_python,
+)
 from .enums import (
+    ConnectionStatus,
     CurrentOperationMode,
     DeviceType,
     DHWControlTypeFlag,
@@ -24,11 +31,17 @@ from .enums import (
     TemperatureType,
     TempFormulaType,
     UnitType,
+    VolumeCode,
 )
 from .field_factory import (
     signal_strength_field,
     temperature_field,
 )
+from .temperature import (
+    HalfCelsius,
+    deci_celsius_to_fahrenheit,
+    half_celsius_to_fahrenheit,
+)
 
 _logger = logging.getLogger(__name__)
 
@@ -37,55 +50,36 @@ _logger = logging.getLogger(__name__)
 # Conversion Helpers & Validators
 # ============================================================================
 
-
-def _device_bool_validator(v: Any) -> bool:
-    """Convert device boolean flag (2=True, 1=False)."""
-    return bool(v == 2)
-
-
-def _div_10_validator(v: Any) -> float:
-    """Divide by 10."""
-    return float(v) / 10.0 if isinstance(v, (int, float)) else float(v)
-
-
-def _half_celsius_to_fahrenheit(v: Any) -> float:
-    """Convert half-degrees Celsius to Fahrenheit."""
-    if isinstance(v, (int, float)):
-        return (float(v) / 2.0 * 9 / 5) + 32
-    return float(v)
+# Reusable Annotated types for conversions
+DeviceBool = Annotated[bool, BeforeValidator(device_bool_to_python)]
+CapabilityFlag = Annotated[bool, BeforeValidator(device_bool_to_python)]
+Div10 = Annotated[float, BeforeValidator(div_10)]
+HalfCelsiusToF = Annotated[float, BeforeValidator(half_celsius_to_fahrenheit)]
+DeciCelsiusToF = Annotated[float, BeforeValidator(deci_celsius_to_fahrenheit)]
+TouStatus = Annotated[bool, BeforeValidator(device_bool_to_python)]
+TouOverride = Annotated[bool, BeforeValidator(tou_override_to_python)]
+VolumeCodeField = Annotated[
+    VolumeCode, BeforeValidator(enum_validator(VolumeCode))
+]
+ConnectionStatusField = Annotated[
+    ConnectionStatus, BeforeValidator(enum_validator(ConnectionStatus))
+]
 
 
 def fahrenheit_to_half_celsius(fahrenheit: float) -> int:
-    """Convert Fahrenheit to half-degrees Celsius (for device commands)."""
-    celsius = (fahrenheit - 32) * 5 / 9
-    return round(celsius * 2)
+    """Convert Fahrenheit to half-degrees Celsius (for device commands).
 
+    Args:
+        fahrenheit: Temperature in Fahrenheit.
 
-def _deci_celsius_to_fahrenheit(v: Any) -> float:
-    """Convert decicelsius (tenths of Celsius) to Fahrenheit."""
-    if isinstance(v, (int, float)):
-        return (float(v) / 10.0 * 9 / 5) + 32
-    return float(v)
+    Returns:
+        Raw device value in half-Celsius format.
 
-
-def _tou_status_validator(v: Any) -> bool:
-    """Convert TOU status (0=False, 1=True)."""
-    return bool(v == 1)
-
-
-def _tou_override_validator(v: Any) -> bool:
-    """Convert TOU override status (1=True, 2=False)."""
-    return bool(v == 1)
-
-
-# Reusable Annotated types for conversions
-DeviceBool = Annotated[bool, BeforeValidator(_device_bool_validator)]
-CapabilityFlag = Annotated[bool, BeforeValidator(_device_bool_validator)]
-Div10 = Annotated[float, BeforeValidator(_div_10_validator)]
-HalfCelsiusToF = Annotated[float, BeforeValidator(_half_celsius_to_fahrenheit)]
-DeciCelsiusToF = Annotated[float, BeforeValidator(_deci_celsius_to_fahrenheit)]
-TouStatus = Annotated[bool, BeforeValidator(_tou_status_validator)]
-TouOverride = Annotated[bool, BeforeValidator(_tou_override_validator)]
+    Example:
+        >>> fahrenheit_to_half_celsius(140.0)
+        120
+    """
+    return int(HalfCelsius.from_fahrenheit(fahrenheit).raw_value)
 
 
 class NavienBaseModel(BaseModel):
@@ -160,7 +154,7 @@ class DeviceInfo(NavienBaseModel):
     additional_value: str = ""
     device_type: DeviceType | int = DeviceType.NPF700_WIFI
     device_name: str = "Unknown"
-    connected: int = 0
+    connected: ConnectionStatusField = ConnectionStatus.DISCONNECTED
     install_type: str | None = None
 
 
@@ -753,21 +747,33 @@ class DeviceFeature(NavienBaseModel):
 
     country_code: int = Field(
         description=(
-            "Country/region code where device is certified for operation "
-            "(1=USA, complies with FCC Part 15 Class B)"
+            "Country/region code where device is certified for operation. "
+            "Device-specific code defined by Navien. "
+            "Example: USA devices report code 3. Earlier project "
+            "documentation incorrectly listed code 1 for USA; field "
+            "observations of production devices confirm that code 3 is "
+            "the correct value."
         )
     )
     model_type_code: UnitType | int = Field(
-        description="Model type identifier: NWP500 series model variant"
+        description=(
+            "Model type identifier: Maps to UnitType enum "
+            "(e.g., NPF=513 for heat pump water heater). "
+            "Identifies the device family and available capabilities"
+        )
     )
     control_type_code: int = Field(
         description=(
-            "Control system type: "
-            "Advanced digital control with LCD display and WiFi"
+            "Control system type identifier: Specifies the version of the "
+            "digital control system (LCD display, WiFi, firmware variant). "
+            "Device-specific numeric code"
         )
     )
-    volume_code: int = Field(
-        description="Tank nominal capacity: 50, 65, or 80 gallons",
+    volume_code: VolumeCodeField = Field(
+        description=(
+            "Tank nominal capacity: 50 gallons (code 1), 65 gallons (code 2), "
+            "or 80 gallons (code 3)"
+        ),
         json_schema_extra={"unit_of_measurement": "gal"},
     )
     controller_sw_version: int = Field(
@@ -814,8 +820,9 @@ class DeviceFeature(NavienBaseModel):
     )
     recirc_model_type_code: int = Field(
         description=(
-            "Recirculation module model identifier - "
-            "specifies installed recirculation system variant"
+            "Recirculation module model identifier: Specifies the type and "
+            "capabilities of the installed recirculation system. "
+            "Device-specific numeric code (0 if recirculation not installed)"
         )
     )
     controller_serial_number: str = Field(
